<?php
/*  Copyright 2002-2003 Patrick R. Michaud (pmichaud@pobox.com)
    This file is part of PmWiki; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License
    (at your option) any later version.  See pmwiki.php for full details.

    This file adds the capability to perform reference counts on
    pages in the PmWiki database.  Simply use "?action=refcount" to
    bring up the reference count results.  The output is a table
    where each row of the table contains a page name or link reference,
    the number of (non-RecentChanges) pages that contain links to the
    page, the number of RecentChanges pages with links to the page, and
    the total number of references in all pages.

    Adding "&expand=y" to the URL causes each entry to be followed by
    a list of the pages containing references to the link or page.

    This script can be activated by placing 
	include_once("scripts/refcount.php");
    in the local.php file.
*/

if (!isset($PageRefCountFmt)) 
  $PageRefCountFmt = "<h1>Reference Count Results</h1><p>";
if ($action=='refcount') { HandleRefCount(); EndHTML(); exit(); }

function HandleRefCount() {
  global $WikiLibDirs,$WikiDir,$GroupNamePattern,$PageTitlePattern,
    $PageRefCountFmt,$WikiWordPattern,$FreeLinkPattern,$UrlPathPattern,
    $HTTP_GET_VARS;
  $expand = $HTTP_GET_VARS['expand'];
  if (!isset($WikiLibDirs)) $WikiLibDirs = array($WikiDir,"wikilib.d");
  StartHTML("Reference count results");
  echo FmtPageName($PageRefCountFmt,NULL);
  foreach($WikiLibDirs as $d) {
    $dp = opendir($d);
    if (!$dp) continue;
    while (($pagename = readdir($dp))!==false) {
      if ($pagelist[$pagename]) continue;
      if (!preg_match("/^$GroupNamePattern\.$PageTitlePattern\$/",$pagename))
        continue;
      $pagelist[$pagename]++;  $ref=array();
      $page = ReadPage($pagename);
      $text = preg_replace("/\\[\\=.*?\\=\\]/",' ',$page['text']);
      $text = preg_replace("/\\w+:($UrlPathPattern)/",' ',$text);
      if (!preg_match_all("/(($GroupNamePattern)[\\/.])?(($WikiWordPattern)|($FreeLinkPattern))/",$text,$match)) continue;
      for($i=0;$i<count($match[0]);$i++) {
        $ref[FmtWikiLink('',$match[0][$i],NULL,'PageName',$pagename)]++;
      }
      $rc = preg_match('/RecentChanges$/',$pagename);
      foreach($ref as $r=>$c) {
        $tref[$r]['tot'] += $c;
        if ($rc) $tref[$r]['rc']++;
        else { $tref[$r]['page']++; $pref[$r][$pagename]++; }
      }
    }
    closedir($dp);
  }
  uasort($tref,'RefCountCmp');
  echo "<table width='100%'><tr><th>Name</th><th>ReferringPages</th>
    <th>RecentChanges</th><th>TotalRefs</th></tr>";
  reset($tref);
  foreach($tref as $p=>$c) {
    echo "<tr><td valign='top'>",FmtWikiLink('',$p,NULL);
    if ($expand && is_array($pref[$p])) {
      foreach($pref[$p] as $pr=>$pc) echo "<dd>",FmtWikiLink('',$pr,NULL);
    }
    echo "</td>";
    echo "<td align='center' valign='top'>",
      FmtPageName("<a href='\$ScriptUrl?action=search&amp;text=\$Title_'>",$p),
      $tref[$p]['page']+0,"</a></td>";
    echo "<td align='center' valign='top'>",$tref[$p]['rc'],"</td>";
    echo "<td align='center' valign='top'>",$tref[$p]['tot'],"</td>";
    echo "</tr>";
  }
  echo "</table>";
  EndHTML();
}

function RefCountCmp($ua,$ub) {
  if ($ua['page']!=$ub['page']) return $ub['page']-$ua['page'];
  if ($ua['rc']!=$ub['rc']) return $ub['rc']-$ua['rc'];
  return $ub['tot']-$ua['tot'];
}
  
?>
